#ifndef RUINA_STATE_UPDATER_HH
#define RUINA_STATE_UPDATER_HH

#include "stateupdater.hh"

template <class ScalarVector, class Vector>
class RuinaStateUpdater : public StateUpdater<ScalarVector, Vector> {
public:
  RuinaStateUpdater(ScalarVector _logState_initial,
                    Dune::BitSetVector<1> const &_nodes, double _L);

  void virtual nextTimeStep();
  void virtual setup(double _tau);
  void virtual solve(Vector const &velocity_field);
  void virtual extractLogState(ScalarVector &);

private:
  ScalarVector logState_o;
  ScalarVector logState;
  Dune::BitSetVector<1> const &nodes;
  double const L;
  double tau;
};

template <class ScalarVector, class Vector>
RuinaStateUpdater<ScalarVector, Vector>::RuinaStateUpdater(
    ScalarVector _logState_initial, Dune::BitSetVector<1> const &_nodes,
    double _L)
    : logState(_logState_initial), nodes(_nodes), L(_L) {}

template <class ScalarVector, class Vector>
void RuinaStateUpdater<ScalarVector, Vector>::nextTimeStep() {
  logState_o = logState;
}

template <class ScalarVector, class Vector>
void RuinaStateUpdater<ScalarVector, Vector>::setup(double _tau) {
  tau = _tau;
}

template <class ScalarVector, class Vector>
void RuinaStateUpdater<ScalarVector, Vector>::solve(
    Vector const &velocity_field) {
  for (size_t i = 0; i < nodes.size(); ++i) {
    if (not nodes[i][0])
      continue;

    double const VoL = velocity_field[i].two_norm() / L;
    // (exp(-tV) - 1)*log(V) + alpha_old*exp(-tV)
    logState[i] =
        (VoL <= 0) ? logState_o[i] : std::expm1(-tau * VoL) * std::log(VoL) +
                                         logState_o[i] * std::exp(-tau * VoL);
  }
}

template <class ScalarVector, class Vector>
void RuinaStateUpdater<ScalarVector, Vector>::extractLogState(
    ScalarVector &_logState) {
  _logState = logState;
}

#endif
